source("defs.R")

if (file.exists("registry")) {
  if (OVERWRITE) {
    unlink("registry_large", recursive = TRUE)
    reg = makeExperimentRegistry(seed = 123L,
      packages = c("mboost", "stabs", "mvtnorm"))
  } else {
    reg = loadRegistry("registry")
  }
} else {
  reg = makeExperimentRegistry("registry_large", seed = 123L,
    packages = c("mboost", "stabs", "mvtnorm")) 
}


reg$default.resources = list(walltime = WALLTIME, memory = MEMORY, ntasks = NTASKS)
reg$max.concurrent.jobs = 1000L

#### create problems ####

for (p in P) {
  i = 1
  for (n in N) {
    j = 1
    for (pinf in PINF) {
      k = 1
      addProblem(name = paste0("P", p, "N", n, "Inf", pinf), seed = as.numeric(paste0(i,j,k)),
        data = list(p = p, n = n, pinf = pinf, toep = TOEPBASE),
        fun = function(data, job){
          p = data$p
          n = data$n
          pinf = data$pinf
          toep = data$toep
          
          sigma = toeplitz(sapply(seq_len(p) - 1, function(x) toep^x))
          data = data.frame(rmvnorm(n = n + 5000, sigma = sigma))
          
          betas = runif(n = pinf, min = -1, max = 1)
          xinf = sample(colnames(data), size = pinf)
          eta = as.matrix(data[, xinf]) %*% betas
          y = rbinom(n, 1, round(exp(eta) / (1 + exp(eta))))
          data$y = y
          
          train = data[1:n,]
          test = data[(n+1):nrow(data), ]
          
          # irrepresentable condition(
          x = as.matrix(train)
          C = (t(x) %*% x)/n
          ind = which(colnames(train) %in% xinf)
          nICviolation = sum((abs(C[-ind, ind] %*% solve(C[ind, ind]) %*% sign(betas))) >= 1)
          
          list(train = train, test = test, xinf = xinf, beta = beta, nICviolation = nICviolation)
        })
      k = k + 1
    }
    j = j + 1
  }
  i = i + 1
}

#### create algorithms ####

probing = function(job, data, instance) {
  
  train = instance$train
  test = instance$test
  xinf = instance$xinf
  
  time = Sys.time()
  mod = glmboost(y ~ ., data = train, control = boost_control(mstop = 500L), probe = TRUE)
  runtime = Sys.time() - time
  units(runtime) = "secs"
  
  # tpr & fpr
  sel = unique(variable.names(mod)[selected(mod)])
  tpr = length(intersect(xinf, sel))/length(xinf)
  fpr =  length(setdiff(sel, xinf)) / (length(variable.names(mod)) - length(xinf) - 1)
  
  #Prediction error
  preds = round(predict(mod, newdata = test, type = "response"))
  mmce = mean(preds[, 1] != test$y)
  
  mod2 = glm(as.formula(paste("y ~", paste(sel, collapse = "+"))), data = train, family = binomial)
  preds = round(predict(mod2, newdata = test, type = "response"))
  mmce_mle = mean(preds != test$y)
  
  list(tpr = tpr, fpr = fpr, runtime = runtime, mmce = mmce, mmce_mle = mmce_mle, mstop = mstop(mod), nICviolation = instance$nICviolation)
}

crossval = function(job, data, instance) {
  
  train = instance$train
  test = instance$test
  xinf = instance$xinf
  
  time = Sys.time()
  mod = glmboost(y ~ ., data = train, control = boost_control(mstop = 5 * nrow(train)))
  opt = cvrisk(mod, papply = lapply)
  mstop(mod) = mstop(opt)
  runtime = Sys.time() - time
  units(runtime) = "secs"
  
  # tpr & fpr
  sel = unique(variable.names(mod)[selected(mod)])
  tpr = length(intersect(xinf, sel))/length(xinf)
  fpr =  length(setdiff(sel, xinf)) / (length(variable.names(mod)) - length(xinf) - 1)
  
  #Prediction error
  preds = round(predict(mod, newdata = test, type = "response"))
  mmce = mean(preds[, 1] != test$y)

  mod2 = glm(as.formula(paste("y ~", paste(sel, collapse = "+"))), data = train, family = binomial)
  preds = round(predict(mod2, newdata = test, type = "response"))
  mmce_mle = mean(preds != test$y)
  
  list(tpr = tpr, fpr = fpr, runtime = runtime, mmce = mmce, mmce_mle = mmce_mle, mstop = mstop(mod), nICviolation = instance$nICviolation)
}


stabilitySelection = function(job, data, instance, pi, pfer) {
  
  train = instance$train
  test = instance$test
  xinf = instance$xinf
  
  #run stabsel
  time = Sys.time()
  mod = glmboost(y ~., data = train, control = boost_control(mstop = 5 * nrow(train)))
  res = stabsel(mod, cutoff = pi, PFER = pfer, papply = lapply)
  runtime = Sys.time() - time
  units(runtime) = "secs"
  
  # tpr & fpr
  sel = names(selected(res))
  
  if (length(sel) == 0) {
    return(list(tpr = 0, fpr = 0, runtime = runtime, mmce = 0.5, mmce_mle = 0.5, mstop = 0, nICviolation = instance$nICviolation))
  }
  
  tpr = length(intersect(xinf, sel))/length(xinf)
  fpr = length(setdiff(sel, xinf)) / (length(variable.names(mod)) - length(xinf) - 1)
  
  
  #Prediction Error
  #mle = glm(as.formula(paste("y ~", names(selected(res)))), data = train, family = binomial())
  mstop(mod) = max(1, which.min(selected(mod) %in% selected(res)) - 1)
  preds = round(predict(mod, newdata = test, type = "response"))
  mmce = mean(preds[, 1] != test$y)
  
  mod2 = glm(as.formula(paste("y ~", paste(sel, collapse = "+"))), data = train, family = binomial)
  preds = round(predict(mod2, newdata = test, type = "response"))
  mmce_mle = mean(preds != test$y)
  
  list(tpr = tpr, fpr = fpr, runtime = runtime, mmce = mmce, mmce_mle = mmce_mle, mstop = mstop(mod), nICviolation = instance$nICviolation)
}


addAlgorithm(reg = reg, name = "probing", fun = probing)
addAlgorithm(reg = reg, name = "crossval", fun = crossval)
addAlgorithm(reg = reg, name = "stabsel", fun = stabilitySelection)

addExperiments(reg = reg, algo.designs = list(
  probing = data.frame(),
  stabsel = expand.grid(pi = PI, pfer = PFER),
  crossval = data.frame()), 
  repls = REPLICATIONS)

summarizeExperiments()
submitJobs()
